use std::net::{TcpListener, TcpStream};
use std::os::unix::net::{UnixListener, UnixStream};
use std::io::prelude::*;
use std::result;
use crate::conf;
use std::sync::{Arc, RwLock};
use std::path::{Path, PathBuf};

pub struct ConfSer{
    path: String,
    listener: UnixListener,
}

impl ConfSer {
    fn new(path: String) -> ConfSer{
        ConfSer{
            path: path.clone(),
            listener: UnixListener::bind(path).unwrap(),
        }
    }
}

impl Drop for ConfSer {
    fn drop(&mut self) {
        // There's no way to return a useful error here
        let _ = std::fs::remove_file(self.path.clone()).unwrap();
        println!("> Dropping {}", self.path);
    }
}

pub fn server_conf(conf: Arc<RwLock<conf::Config>>){
    //let listener = UnixListener::bind("mineproxy").unwrap();
    let ser = ConfSer::new(String::from("mineproxy"));

    for stream in ser.listener.incoming() {
        match stream{
        Ok(s) => {
                ConfCon{
                    conf: conf.clone(),
                    stream: s,
                    buf: [1; 256],
                }.process_reques();
            },
        Err(_e) => println!("{}",_e),
        }
    }

}

pub struct ConfCon {
    conf: Arc<RwLock<conf::Config>>,
    stream: UnixStream,
    buf:[u8; 256],
}

impl ConfCon {
    fn process_reques(&mut self) {
        //let mut buf: [u8; 256] = [1; 256];
        match self.stream.read(&mut self.buf){
            Ok(len) => self.check_state(len),
            Err(e) => println!("Fatal: {}", e),
        }
    }

    fn read_domain_info(&mut self) ->
            Result<conf::ServerData, std::io::Error>{
        let mut len = self.stream.read(&mut self.buf)?;
        let domain = String::from_utf8(self.buf[0 .. len].to_vec()).unwrap();
        len = self.stream.read(&mut self.buf)?;
        let ip = String::from_utf8(self.buf[0 .. len].to_vec()).unwrap();
        Ok(conf::ServerData{
            domain: domain,
            ip: ip,
        })
    }

    fn check_state(&mut self, len: usize) {
        let var = String::from_utf8(self.buf[0 .. len].to_vec()).unwrap();
        match var.as_str(){
            "add" => {
                match self.read_domain_info() {
                    Ok(sd) => println!("domain {}",sd.ip),
                    Err(e) => println!("e: {}",e),
                }
            },
            "dell" => println!("del"),
            "mod" => println!("mod"),
            _=> println!("no recognice option: {}", var),
        }
    }
}

